#ifndef USERLIST_H
#define USERLIST_H

#include "user.h"

#include <QObject>

class UserList : public QObject
{
    Q_OBJECT
public:
    UserList(const QVector<User> &list);

    QVector<User> getList() const;
    void setList(const QVector<User> &newList);

    struct iteratable
    {
        using iterator_category = std::forward_iterator_tag;
        using difference_type = std::ptrdiff_t;
        using value_type = User;
        using pointer = User *;   // or also value_type*
        using reference = User&; // or also value_type&

        iteratable(User *begin)
            : ptr{begin}
        {}

        reference operator*() const { return *ptr; }
        pointer operator->() { return ptr; }

        // Prefix increment
        iteratable& operator++() { ptr++; return *this; }

        // Postfix increment
        iteratable operator++(int) { iteratable tmp = *this; ++(*this); return tmp; }

        friend bool operator== (const iteratable& a, const iteratable& b) { return a.ptr == b.ptr; };
        friend bool operator!= (const iteratable& a, const iteratable& b) { return a.ptr != b.ptr; };

    private:
        User* ptr;
    };
    struct const_interatable
    {
        using iterator_category = std::forward_iterator_tag;
        using difference_type = std::ptrdiff_t;
        using value_type = const User;
        using pointer = const User *;   // or also value_type*
        using reference = const User&; // or also value_type&

        const_interatable(const User *begin)
            : ptr{begin}
        {}

        reference operator*() const { return *ptr; }
        pointer operator->() { return ptr; }

        // Prefix increment
        const_interatable& operator++() { ptr++; return *this; }

        // Postfix increment
        const_interatable operator++(int) { const_interatable tmp = *this; ++(*this); return tmp; }

        friend bool operator== (const const_interatable& a, const const_interatable& b) { return a.ptr == b.ptr; };
        friend bool operator!= (const const_interatable& a, const const_interatable& b) { return a.ptr != b.ptr; };

    private:
        const User* ptr;
    };

    iteratable begin() { return &*list.begin(); }
    iteratable end()   { return &*list.end(); }

    const_interatable begin() const { return &*list.begin(); }
    const_interatable end()   const { return &*list.end(); }

private:
    QVector<User> list;
};

#endif // USERLIST_H
